
#include "stdh.h"
#include "GFont.h"

#define MAX_CHARTERS  40  // pocet pismenek ve fontu
#define SPACE         25  // sirka mezery (ma ID MAX_CHARTERS)
#define CHARSPACE      5  // vzdalenost mezi pismenky

BITMAP *my_font[MAX_CHARTERS];

typedef struct {
	char c;
	char file[80];
} st_fontdata;

const st_fontdata Fparams[MAX_CHARTERS] = {
	{'A', "gfx\\font\\a.bmp"},
	{'B', "gfx\\font\\b.bmp"},
	{'C', "gfx\\font\\c.bmp"},
	{'D', "gfx\\font\\d.bmp"},
	{'E', "gfx\\font\\e.bmp"},
	{'F', "gfx\\font\\f.bmp"},
	{'G', "gfx\\font\\g.bmp"},
	{'H', "gfx\\font\\h.bmp"},
	{'I', "gfx\\font\\i.bmp"},
	{'J', "gfx\\font\\j.bmp"},
	{'K', "gfx\\font\\k.bmp"},
	{'L', "gfx\\font\\l.bmp"},
	{'M', "gfx\\font\\m.bmp"},
	{'N', "gfx\\font\\n.bmp"},
	{'O', "gfx\\font\\o.bmp"},
	{'P', "gfx\\font\\p.bmp"},
	{'Q', "gfx\\font\\q.bmp"},
	{'R', "gfx\\font\\r.bmp"},
	{'S', "gfx\\font\\s.bmp"},
	{'T', "gfx\\font\\t.bmp"},
	{'U', "gfx\\font\\u.bmp"},
	{'V', "gfx\\font\\v.bmp"},
	{'W', "gfx\\font\\w.bmp"},
	{'X', "gfx\\font\\x.bmp"},
	{'Y', "gfx\\font\\y.bmp"},
	{'Z', "gfx\\font\\z.bmp"},

	{'0', "gfx\\font\\0.bmp"},
	{'1', "gfx\\font\\1.bmp"},
	{'2', "gfx\\font\\2.bmp"},
	{'3', "gfx\\font\\3.bmp"},
	{'4', "gfx\\font\\4.bmp"},
	{'5', "gfx\\font\\5.bmp"},
	{'6', "gfx\\font\\6.bmp"},
	{'7', "gfx\\font\\7.bmp"},
	{'8', "gfx\\font\\8.bmp"},
	{'9', "gfx\\font\\9.bmp"},

	{'<', "gfx\\font\\zLEVO.bmp"},
	{'>', "gfx\\font\\zPRAVO.bmp"},
	{'-', "gfx\\font\\zPOMLCKA.bmp"},
	{'.', "gfx\\font\\zTECKA.bmp"},
};

bool LoadFontGFX() 
{
	char s[80];
	for(int i = 0; i < MAX_CHARTERS; i++) {
		sprintf(s, Fparams[i].file);
		if (!(my_font[i] = hload_shadow_bitmap(s))) {
			allegro_message("Misssigne file %s", s);
			return false;
		}
	}
	return true;
}

void DestroyFont()
{
	for(int i = 0; i < MAX_CHARTERS; i++) 
		if (my_font[i] != NULL) {
			destroy_bitmap(my_font[i]);
			my_font[i] = NULL;
		}
}

int GetTextWidth(char *c)
{
	int w = 0;
	int i;
	while(*c != '\0') {
		if ((i = GetFontID(*c)) != -1) {
			if (i != MAX_CHARTERS) 
				w += my_font[i]->w + CHARSPACE;
			else w += SPACE;
		}
		c++;
	}
	return w;
}

// Vrati id znaku v Fparams, jinak vraci -1
int GetFontID(char c)
{
	if (c == ' ') return MAX_CHARTERS;
	for (int i = 0; i < MAX_CHARTERS; i++)
		if (Fparams[i].c == c) break;
	if (i < MAX_CHARTERS) return i;
	return -1;
}


void DrawMyFontCenter(BITMAP *dst, int x, int y, int color, char *c)
{
	// font v 8BPP neni podporovan !!
	if (hscreen.m_bpp == 8) {
		textprintf_centre(dst, font, x, y, color, c);
		return;
	}

	x -=  GetTextWidth(c)/2;
	int i;
	while(*c != '\0') {
		if ((i = GetFontID(*c)) != -1) {
			if (i != MAX_CHARTERS) {
				BITMAP *colored = create_bitmap(my_font[i]->w, my_font[i]->h);
				clear_to_color(colored, color);
				hmasked_sprite(dst, colored, my_font[i], x, y);
				destroy_bitmap(colored);
				x += my_font[i]->w + CHARSPACE;
			}
			else x += SPACE;
		}
		c++;
	}
}
