// GMap.cpp: implementation of the GMap class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GMap.h"
#include "GView.h"
#include "GRun.h"


/*////////////////////////////////////////////////////////////////////////////////
  Konstruktor
*/
GMap::GMap()
{
	m_bGround = NULL;
	m_bWalk   = NULL;

	int i;

	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		m_strely[i].m_active = false;
	m_strel = 0;

	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++)
			m_power[i] = NULL;

	m_nextpwr = 0;


	m_sx = 1280;
	m_sy = 1280;

	m_mode = 0;
}

/*////////////////////////////////////////////////////////////////////////////////
  Destruktor
*/
GMap::~GMap()
{
	DestroyArrays();
}

/*////////////////////////////////////////////////////////////////////////////////
  Smaze vsechna data a pripravi se na nove zadavani
  road   :  pocet bodu silnice
  object :  pocet objektu
*/
void GMap::Reset(int X, int Y)
{
	DestroyArrays();
	m_sx = X;
	m_sy = Y;
}

/*////////////////////////////////////////////////////////////////////////////////
  Oznamuje, ze byla hotova inicializace (pri nacitani ze souboru)
*/
void GMap::InitDone()
{
	if (m_mode == TYPE_LARGE)
		FillLargeGround();
}
/*////////////////////////////////////////////////////////////////////////////////
  Znici alokovane bitmapy
*/


/*////////////////////////////////////////////////////////////////////////////////
  Kresli mapu - podklad (travu a silnice)
  CX  :  x-ova souradnice kamery
  CY  :  y-ova souradnice kamery
*/
void GMap::Draw(int CX, int CY, bool stin)
{
	//TODO: dodelat mapu
	int i;
	// podklad
	if (m_mode == 0)
		blit(m_bGround, hscreen.m_back, (CX - DF_X/2), (CY - DF_Y/2), 0, 0, DF_X, DF_Y);
	else 
		blit(m_bWalk, hscreen.m_back, (CX - DF_X/2), (CY - DF_Y/2), 0, 0, DF_X, DF_Y);

	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++)
		if (m_power[i] != NULL) m_power[i]->Draw(CX, CY);

	// kresleni strel
	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		if(m_strely[i].m_active) m_strely[i].Draw(CX, CY, stin);
}

/*////////////////////////////////////////////////////////////////////////////////
  Kresli mapu - objekty
  CX  :  x-ova souradnice kamery
  CY  :  y-ova souradnice kamery
  hi  :  vyskova hladina 
*/
//DEL void GMap::DrawObj(int CX, int CY, int hi, bool stin)
//DEL {
//DEL 	for( int i = 0; i < m_cobjects; i++)
//DEL 		if (m_objects[i] != NULL) m_objects[i]->Draw(hscreen.m_back, CX, CY, hi, stin);
//DEL }

/*////////////////////////////////////////////////////////////////////////////////
  Nacte graficka data
*/
bool GMap::LoadGFX()
{
	// grafika se nacita az pri zadani levelu

	return true;
}
/*////////////////////////////////////////////////////////////////////////////////
  Pohne vsemi objekty
*/
void GMap::Move()
{
	int i;

	if(--m_nextpwr <= 0) 
		NewPowerUp();  // jak dlouho zbyva do pridani dalsiho powerupu


	m_strelA = 0;
	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		if (m_strely[i].m_active) {
			m_strely[i].Move(this);
			m_strelA++;
		}
}

/*////////////////////////////////////////////////////////////////////////////////
  Rusi pole m_objects
*/
void GMap::DestroyArrays()
{
	int i;

	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		m_strely[i].m_active = false;
	m_strel = 0;

	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++)
		if (m_power[i] != NULL) {
			delete m_power[i];
			m_power[i] = NULL;
		}
}

/*////////////////////////////////////////////////////////////////////////////////
  Testuje narazy - bere vsechna auticka a zkousi je na vsechny objekty
*/

void GMap::FillLargeGround(){

}

/*////////////////////////////////////////////////////////////////////////////////
  Nici bitmapy 
*/
void GMap::Destroy()
{
	if (m_bGround != NULL)
		destroy_bitmap(m_bGround);
	m_bGround = NULL;
	if (m_bGroundZal != NULL)
		destroy_bitmap(m_bGroundZal);
	m_bGroundZal = NULL;
	if (m_bWalk != NULL)
		destroy_bitmap(m_bWalk);
	m_bWalk = NULL;

	DestroyArrays();
}

/*////////////////////////////////////////////////////////////////////////////////
  prida novou strelu
*/
void GMap::NewStrela(int x, int y, int angle, int ID, int who, double speed, int zbranID)
{
	m_strel++;
	int i;
	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		if(!m_strely[i].m_active) break;
	if (i == MAX_STREL_ON_MAP) return;
	m_strely[i].Init(x,y,angle,ID,who, speed, zbranID);
}

/*////////////////////////////////////////////////////////////////////////////////
	Zkusi zasahy do hracu
*/
void GMap::ShootHits(GRun *run)
{
	int i, who = 0, ID;
	for(i = 0; i < MAX_STREL_ON_MAP; i++)
		if (m_strely[i].m_active) {
			who = m_strely[i].m_who;
			ID  = m_strely[i].m_ID;
			for( int k = 0; k < run->m_soldiers; k++) {
				if (!m_strely[i].m_active) break;
				if (Hit(&run->m_sold[k], &m_strely[i], run)) {
					if (!run->m_multi) {
						InsertKilled(run->m_sold[k].m_x, run->m_sold[k].m_y, Nparams[ID].dead, run->m_sold[k].m_angle);
						if((++run->m_sold[who].m_frags)%10 == 0) 
							run->m_view.AddFragsMess(who, run);
					}
					else 
					if (k == 0) {
						run->m_net.SendFrag(who, Nparams[ID].dead);
						InsertKilled(run->m_sold[k].m_x, run->m_sold[k].m_y, Nparams[ID].dead, run->m_sold[k].m_angle);
					}
				}
			}
		}
}

/*////////////////////////////////////////////////////////////////////////////////
	Zda se dana strela trefila
*/
bool GMap::Hit(GSoldier *sold, GStrela *strela, GRun *run)
{
	GStrela *str = strela;
	bool ret = false;

	// aby hrac nezabil sam sebe
	if (sold->m_health <= 0) return false;
	if (str->m_who == sold->m_index) return false;


	// test narazu
	// testuje se pozice strely a naledujici a predchazenici v pohybu
	if (!strela->Hit(sold->m_x, sold->m_y, sold->m_dist)) return false;

	// zasah
	if ((ret = sold->HitBy(str->m_ID, str->GetDemage(this),str->m_who)) == true) {
		// vojak byl zabit - tak to oznamime
		run->m_view.AddKillMessage(str->m_who, sold->m_index, run);
	}

	// umazani strely 
	strela->m_active = false;

	return ret;
}

/*////////////////////////////////////////////////////////////////////////////////
	Prida do hry jeden dalsi bonus
*/
void GMap::NewPowerUp()
{
	int i;
	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++) 
		if (m_power[i] == NULL) break;
	if (i == MAX_POWERUPS_ON_MAP) return;

	m_power[i] = new GPowerUp();

	int x,y;
	GetFreePos(&x, &y, 20);

	m_power[i]->Init(x, y, rand()%MAX_POWERUPS);

	m_nextpwr = 5+rand()%10;
}

/*////////////////////////////////////////////////////////////////////////////////
	Zajistuje zbirani powerupu
*/
void GMap::PickPowerUps(GRun *run)
{
	int i,j;
	for(i = 0; i < MAX_POWERUPS_ON_MAP; i++)
		if (m_power[i] != NULL)
			for( int k = 0; k < run->m_soldiers; k++) {
				if (m_power[i] == NULL) break;
				j = m_power[i]->m_ID;
				if (m_power[i]->Picked(&run->m_sold[k], &m_power[i])) {
					if (k == run->m_monsol) run->m_view.AddBonMessage(j);
					break;
				}
			}
}

/*////////////////////////////////////////////////////////////////////////////////
	Vlozi mrtvolu vojaka na zem
*/
void GMap::InsertKilled(int x, int y, int ID, int angle)
{
	rotate_sprite(m_bGround, b_deads[ID],  x-20, y-20, itofix(-angle));
}

/*////////////////////////////////////////////////////////////////////////////////
	Vrati volnou pozici na mape
*/
void GMap::GetFreePos(int *x, int *y, int dst)
{
	do {
		*x = rand()%m_sx;
		*y = rand()%m_sy-80;
	} while (!CanWalk(*x, *y));
}

/*////////////////////////////////////////////////////////////////////////////////
	Vrati zda je dane policko pruchodne
*/
bool GMap::CanWalk(int x, int y)
{
	int col = getpixel(m_bWalk, x, y);
	if (col == makecol(255,255,255) ||
		 col == makecol(  0,  0,255)) return true;
	return false;
}

/*////////////////////////////////////////////////////////////////////////////////
	Vrati zda je dana pozice pristrelna
*/
bool GMap::CanShoot(int x, int y)
{
	int col = getpixel(m_bWalk, x, y);
	if (col == -1 || col == 0) return false;
	return true;
}

/*////////////////////////////////////////////////////////////////////////////////
	Cisti plochu od mrtvol
*/
void GMap::ClearGround()
{
	draw_sprite(m_bGround, m_bGroundZal, 0, 0);
}

/*////////////////////////////////////////////////////////////////////////////////
	Zda se da jit danym smerem 
*/
int GMap::CanWalk(int x, int y, double angle, double speed)
{
	int  ret = CAN_WALK_ALL;
//	int  i;

	// tes X souradnice a Y souradnice, ale zvlast
	double cx = cos(angle);
	double sy = sin(angle);

	if ( CanWalkAround(x+cx*speed,y-sy*speed) &&
		  CanWalkAround(x+cx*speed/2,y-sy*speed/2) &&
		  CanWalkAround(x+cx*speed*3/4,y-sy*speed*3/4) &&
		  CanWalkAround(x+cx*speed/4,y-sy*speed/4) ) return CAN_WALK_ALL;
/*
	for (i = 1 ; i < 4; i++) {
		if (!CanWalkAround(x+cx*speed*i/4,y)) ret &= ~CAN_WALK_X;
		if (!CanWalkAround(x,y-sy*speed*i/4)) ret &= ~CAN_WALK_Y;
	}
*/
	if (!CanWalkAround(x+cx*speed/2,y)) ret &= ~CAN_WALK_X;
	if (!CanWalkAround(x,y-sy*speed/2)) ret &= ~CAN_WALK_Y;

	if (!CanWalkAround(x+cx*speed,y)) ret &= ~CAN_WALK_X;
	if (!CanWalkAround(x,y-sy*speed)) ret &= ~CAN_WALK_Y;

	return ret;
}

/*////////////////////////////////////////////////////////////////////////////////
	Vrati zda je mozne strilet danym smerem
*/
bool GMap::CanShoot(int x, int y, double angle, double speed)
{
	// test X souradnice a Y souradnice naraz
	double cx = cos(angle);
	double sy = sin(angle);
	int i;

	for (i = 1 ; i < speed; i += 2)
		if (!CanShoot(x+cx*i,y-sy*i)) return false;

	if (!CanShoot(x+cx*speed,y-sy*speed)) return false;

	return true;
}

/*////////////////////////////////////////////////////////////////////////////////
	nacte danou mapu
*/
bool GMap::LoadMap(int mapa)
{
	Destroy();
	char s[70];

	PALETTE pal;


	sprintf(s, "lev%d.bmp", mapa);
	if (!(m_bGround    = load_bitmap(s, pal))) return false;
	if (!(m_bGroundZal = create_bitmap(m_bGround->w, m_bGround->h))) return false;
	draw_sprite(m_bGroundZal, m_bGround, 0, 0);

	sprintf(s, "lev%db.pcx", mapa);
	if (!(m_bWalk   = load_bitmap(s, pal))) return false;

	m_sx = m_bGround->w;
	m_sy = m_bGround->h;

	m_mapa = mapa;

	return true;
}

/*////////////////////////////////////////////////////////////////////////////////
	Nacte Demo mapu
*/
void GMap::LoadDemo()
{
	Destroy();
	
	PALETTE pal;

	m_bGround    = load_bitmap("demo.bmp", pal);
	m_bGroundZal = create_bitmap(m_bGround->w, m_bGround->h);
	draw_sprite(m_bGroundZal, m_bGround, 0, 0);

	m_bWalk   = create_bitmap(640, 480);
	clear_to_color(m_bWalk, makecol(255,255,255));

	m_sx = 640;
	m_sy = 560;
	
	m_mapa = 0;
}

/*////////////////////////////////////////////////////////////////////////////////
	Vrati zda je na danem miste voda
*/
bool GMap::IsWater(int x, int y)
{
	int col = getpixel(m_bWalk, x, y);
	if (col == makecol(0,0,255)) return true;
	return false;
}

bool GMap::CanWalkAround(int x, int y)
{
	return (CanWalk(x,y)     && 
			  CanWalk(x+1,y)   && CanWalk(x,y+1) &&
			  CanWalk(x-1,y)   && CanWalk(x,y-1) &&
			  CanWalk(x+1,y+1) && CanWalk(x-1,y-1) &&
			  CanWalk(x+1,y-1) && CanWalk(x-1,y+1));
}
