// GNet.cpp: implementation of the GNet class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "GNet.h"
#include "GRun.h"
#include "GSoldier.h"
#include "GMap.h"
#include "GClasses.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

GNet::GNet()
{
	m_sended = 0;
	m_received = 0;
	m_connected = false;
}

GNet::~GNet()
{

}

void GNet::Send(GRun *run)
{
	if (!m_connected) return;
	// posleme hrace
	GSoldier *sold = &run->m_sold[run->m_monsol];
	st_msg_soldier *msg;
	msg = (st_msg_soldier* )m_buf;
	msg->anim      = sold->m_animate;
	msg->angle     = sold->m_angle%256;
	msg->frags     = sold->m_frags   ;
	msg->health    = sold->m_health  ;
	msg->ID        = sold->m_ID      ;
	msg->shield    = sold->m_shield  ;
	msg->x         = sold->m_x       ;
	msg->y         = sold->m_y       ;
	msg->zbran     = sold->m_zbran   ;
	msg->params    = 3 & sold->m_a_zbran;
	msg->type = MESS_SOLDIER;
	msg->strel = 0;

		// posleme strely naseho hrace
	msg++;
	st_msg_strela *msg2 = (st_msg_strela *) msg;
	msg = (st_msg_soldier* )m_buf;
	for (int i = 0; i < MAX_STREL_ON_MAP; i++)
		if (run->m_map.m_strely[i].m_active && run->m_map.m_strely[i].m_who == 0) {
			msg2->angle = run->m_map.m_strely[i].m_angle%256;
			msg2->ID    = run->m_map.m_strely[i].m_ID;
			msg2->x     = run->m_map.m_strely[i].m_x;
			msg2->y     = run->m_map.m_strely[i].m_y;
			msg->strel++;
			msg2++;
		}
	net->Send(DPID_ALLPLAYERS, &m_buf, sizeof(st_msg_soldier)+msg->strel*sizeof(st_msg_strela));	
	m_sended++;
}

void GNet::Receive(GRun *run)
{
	if (!m_connected) return;
	// prijimame zpravy
	while (net->Receive()) {
		m_received++;
		// systemova zprava
		if(net->FromDPID==DPID_SYSMSG) {
			DPMSG_GENERIC *GenMsg=(DPMSG_GENERIC*)net->RBuffer;
			// Je z nas server ;)
			if(GenMsg->dwType==DPSYS_HOST) {
				run->m_server = true;
			}
		}
		// zprava od hrace
		else {
			// Najdeme hrace, ktery nam posila zpravu
			for(int i = 0; i < net->NumPlayers; i++) 
				if(net->Players[i].dpid==net->FromDPID) {
					ProcessMess(i, run);
					break;
				}
		}
	}
}

bool GNet::EnterGame(GRun *run)
{
	if (!m_connected) return true;

	// hledame ostatni hty
	net->ScanForGames(0,false);

	// Uz hra existuje, takze se connectnem
	if (net->NumGames > 0)
	{
		if (!net->JoinGame(0,run->m_sold[0].m_name)) {
			ShutDown();
			return false;
		}
		run->m_server = false;
	}
	else { // Zadna hra jeste neni -> tvorime vlastni novou
		if (!net->CreateGame("Test Game", run->m_sold[0].m_name, MAX_PLAYERS)) {
			ShutDown();
			return false;
		}
		run->m_server = true;
	}

	net->SetPlayerScanEvent(200);

	return true;
}

void GNet::ProcessMess(int from, GRun *run)
{
	if (!m_connected) return;

	if (from <= m_myindex) from++;

	m_frommess = from;
	DPMSG_GENERIC *GenMsg = (DPMSG_GENERIC*) net->RBuffer;
	switch (GenMsg->dwType) {
	case MESS_SOLDIER :
		ProcessSoldierMess(from, run);
		break;
	case MESS_FRAG :
		ProcessFragMess(from, run);
		break;
	case MESS_KILLED :
		ProcesKilledMess(from, run);
		break;

	}
}

void GNet::ProcessSoldierMess(int from, GRun *run)
{
	if (!m_connected) return;

	int i;
	GSoldier *sold = &run->m_sold[from];
	st_msg_soldier *msg = (st_msg_soldier*) net->RBuffer;
	sold->m_animate   = msg->anim;
	sold->m_angle     = msg->angle   ;
	sold->m_frags     = msg->frags   ;
	sold->m_health    = msg->health  ;
	sold->m_ID        = msg->ID      ;
	sold->m_shield    = msg->shield  ;
	sold->m_x         = msg->x       ;
	sold->m_y         = msg->y       ;
	sold->m_zbran     = msg->zbran   ;
	sold->m_a_zbran     = 3 & msg->params;

	// smazeme vsechny strely daneho hrace (budou znova zavedeny)
	run->m_map.DelStrelyFrom(msg->ID);

	msg++;
	st_msg_strela *msg2 = (st_msg_strela *) msg;
	msg = (st_msg_soldier* ) net->RBuffer;
	for(i = 0; i < msg->strel; i++, msg2++){
		run->m_map.NewNetStrela(msg2->x, msg2->y, msg2->angle, msg2->ID, msg->ID);
	}

}

void GNet::ProcessFragMess(int from, GRun *run)
{
	run->m_sold[0].m_frags++;
}

void GNet::UpdateSoldiers(GRun *run)
{
	if (!m_connected) return;

	int i,j;
	for(i = 0; i < net->NumPlayers; i++) 
		if(net->Players[i].dpid == net->LocalDPID) {
			m_myindex   = i;
			break;
		}

	if (run->m_soldiers != net->NumPlayers) {
		for(i = 0; i < net->NumPlayers; i++)
			if (net->Players[i].dpid != net->LocalDPID) {
				j = i;
				if (j < m_myindex) j++;
				strcpy(run->m_sold[j].m_name, net->Players[i].Name);
			}

		run->m_soldiers = net->NumPlayers;
	}
}

void GNet::ShutDown()
{
	if (!m_connected) return;

	net->CloseConnection();
	net->Create(&guid);
}

void GNet::SendFrag(int kdo, int jak)
{
	if (!m_connected) return;
	st_msg_frag    msg;
	st_msg_killed  msg2;
	if (kdo <= m_myindex) kdo--;

	msg.type = MESS_FRAG;
	net->Send(net->Players[kdo].dpid, &msg, sizeof(st_msg_frag));	
	m_sended++;

	msg2.type   = MESS_KILLED;
	msg2.zpusob = jak;
	net->Send(DPID_ALLPLAYERS, &msg2, sizeof(st_msg_killed));
	m_sended++;

}

void GNet::ProcesKilledMess(int from, GRun *run)
{
	if (!m_connected) return;

	st_msg_killed *msg = (st_msg_killed*) net->RBuffer;
	run->m_map.InsertKilled(run->m_sold[from].m_x, run->m_sold[from].m_y, msg->zpusob, run->m_sold[from].m_angle);
}
