// SCar.cpp: implementation of the SCar class.
//
//////////////////////////////////////////////////////////////////////

#include "stdh.h"
#include "GSoldier.h"
#include "GObject.h"
#include "GMap.h"
#include "GRun.h"


//////////////////////////////////////////////////////////////////////////////////
//                          INICIALIZACNI FUNKCE
//////////////////////////////////////////////////////////////////////////////////

/*////////////////////////////////////////////////////////////////////////////////
  konstructor - nuluje promene
*/
GSoldier::GSoldier()
{
	m_x = 0;
	m_y = 0;
	m_angle = 0;
	m_speed = 0;
	m_dist = 10;
}

/*////////////////////////////////////////////////////////////////////////////////
  destruktor
*/
GSoldier::~GSoldier()
{
}

/*////////////////////////////////////////////////////////////////////////////////
  pri novem zavodu - vymaze vse a nasatvi pozici auta 
  index :  index v poli u GRUN
  x,y   :  pozice na hraci plose
*/
void GSoldier::Init(int index, int x, int y, int zbran, bool dorust, int vybaven, GMap *map)
{
	m_zanim    = 0;
	m_tochange = 0;
	m_index = index;
	m_sx     = x;
	m_sy     = y;
	m_frags  = 0;
	m_zbran  = zbran;

	m_a_shield = -1;
	m_a_krev   = -1;

	m_lastkill   = -1;  
	m_lastkilled = -1;

	m_AI.Init();

	RaiseFromDead(map, vybaven);
}

/*////////////////////////////////////////////////////////////////////////////////
  Nastavi pozadavky ridice
  fwd   : zda chce jet vpred
  back  : pro jizdu vzad(brzdu)
  left  : zatoceni wlevo
  right : zatoceni wpravo
*/
void GSoldier::SetKeys(bool fwd, bool back, bool left, bool right, bool side, bool run, bool shoot, bool chzbran, bool liveup)
{
	m_fwd     = fwd;
	m_back    = back;
	m_left    = left;
	m_right   = right;
	m_side    = side;
	m_run     = run;
	m_shoot   = shoot;
	m_chzbran = chzbran;
	m_liveup  = liveup;
}




//////////////////////////////////////////////////////////////////////////////////
//                              GRAFICKE FUNKCE
//////////////////////////////////////////////////////////////////////////////////

/*////////////////////////////////////////////////////////////////////////////////
  Kresli vojaka na danou bitmapu ( zavodni plochu )
  dest - pointer na bitmapu, kam se ma auto nakreslit (velikosti DF_X, DF_X )
  x,y  - souradnice stredu dest vuci trati
*/
void GSoldier::Draw(BITMAP *dest, int x, int y)
{
	// uprava parametru
	x = m_x - x + (DF_X - DF_SOLD_X) / 2;
	y = m_y - y + (DF_Y - DF_SOLD_Y) / 2;

	if (x > DF_X + 40 || x < -40 ||
		 y > DF_Y + 40 || y < -40) return;

	// kresleni vojaka
	if (m_health > 0) {
		rotate_sprite(dest, b_solds[Sparams[m_ID].bindex], x, y, itofix(-m_angle));
		if(b_zbrane[m_zbran][m_zanim] != NULL) 
			rotate_sprite(dest, b_zbrane[m_zbran][m_zanim], x-10, y-10, itofix(-m_angle));
	}
	// trocha krve
	if (m_a_krev >= 0) {
		draw_sprite(dest, b_krev[m_a_krev], x-2, y-2);
		m_a_krev--;
	}
	// malikej stit
	if (m_a_shield >= 0) {
		draw_sprite(dest, b_stit[m_a_shield], x-2, y-2);
		m_a_shield--;
	}

	// napsani jmena
	textout_centre(hscreen.m_back, (FONT *)fonts[FNT_SMALL].dat, m_name, x+DF_SOLD_X/2, y-25, makecol(200,200,0));

#ifdef _DEBUG
	if(m_index == 0) {
		char s[60];
		sprintf(s,"x=%.2f y=%.2f angle=%d frags=%d health=%d stit=%d",m_x, m_y, m_angle, m_frags, m_health, m_shield);
		textout(hscreen.m_back, font, s, 10, 10, makecol(200,200,200));
	}
#endif

}


/*////////////////////////////////////////////////////////////////////////////////
  Pripravi grafika data
*/
bool GSoldier::LoadGFX()
{
	Destroy();

	return true;
}
/*////////////////////////////////////////////////////////////////////////////////
  Znici grafika data
*/
void GSoldier::Destroy()
{
}


//////////////////////////////////////////////////////////////////////////////////
//                              POHYBOVE FUNKCE
//////////////////////////////////////////////////////////////////////////////////

/*////////////////////////////////////////////////////////////////////////////////
   Udela Pohyb 
*/
void GSoldier::Move(GRun *run)
{

	if (m_health <= 0) {
		if (m_liveup) RaiseFromDead(&run->m_map, run->m_maxstrel);
		return;
	}

	int speed = Sparams[m_ID].speed;
	double angle2 = 0;
	if (m_run) speed <<= 1; // nasobeni 2
	if (m_fwd && !m_back) m_speed = speed;
	else if (!m_fwd && m_back) m_speed = -speed;
		  else m_speed = 0;

	// zataceni
	if (!m_side) { 
		if (m_left && !m_right) m_angle += Sparams[m_ID].turning;
		if (!m_left && m_right) m_angle -= Sparams[m_ID].turning;
	}
	else { // chozen ido stran
		if (m_left && !m_right) angle2 = PI2;
		if (!m_left && m_right) angle2 = -PI2;
	}

	m_angleRad = m_angle*DPI/255;

	// vlastni posum souradnic
	if (run->m_map.CanWalk(m_x+cos(m_angleRad) * m_speed,m_y,20))
		m_x += cos(m_angleRad) * m_speed;
	if (run->m_map.CanWalk(m_x,m_y-sin(m_angleRad) * m_speed,20))
		m_y -= sin(m_angleRad) * m_speed;
	if(angle2 != 0){ // chozeni do strany
		if (run->m_map.CanWalk(m_x+cos(m_angleRad+angle2) * speed,m_y,20))
			m_x += cos(m_angleRad+angle2) * speed;
		if (run->m_map.CanWalk(m_x,m_y-sin(m_angleRad+angle2) * speed,20))
			m_y -= sin(m_angleRad+angle2) * speed;
	}

	if (m_shoot && m_nrtf==0 && m_naboje[Zparams[m_zbran].strelaID] > 0) {
		for(int i = 0; i < Nparams[Zparams[m_zbran].strelaID].attime; i++) {
			int angle = m_angle + rand()%(Zparams[m_zbran].angle+1)-Zparams[m_zbran].angle/2;
			run->m_map.NewStrela(m_x+rand()%6-3,m_y+rand()%6-3,angle,Zparams[m_zbran].strelaID, m_index, m_speed, m_zbran);
		}
		m_nrtf = Zparams[m_zbran].rate;
		m_naboje[Zparams[m_zbran].strelaID]--;
		if (Zparams[m_zbran].so) ++m_zanim;
	}
	if (!Zparams[m_zbran].so) ++m_zanim;

	if(m_zanim >= Zparams[m_zbran].anim) m_zanim = 0;


	if (m_x < DF_SOLD_X/2) m_x = DF_SOLD_X/2;
	if (m_y < DF_SOLD_Y/2) m_y = DF_SOLD_Y/2;
	if (m_x > run->m_map.m_sx-DF_SOLD_X/2) { m_x = run->m_map.m_sx-DF_SOLD_X/2; }
	if (m_y > run->m_map.m_sy-DF_SOLD_Y/2-85) { m_y = run->m_map.m_sy-DF_SOLD_Y/2-85; }

	// dorustani veci
	if (++m_nshield > Sparams[m_ID].dshields) {
		m_nshield = 0;
		if(m_shield < Sparams[m_ID].shields) m_shield++;
	}
	if (m_nrtf > 0) m_nrtf--;

	if (m_tochange > 0) m_tochange--;
	if ((m_chzbran == true && m_tochange == 0)||(m_naboje[Zparams[m_zbran].strelaID] <= 0)) {
		if(++m_zbran >= MAX_ZBRANE)
			m_zbran = 0;
		m_tochange = 10;
	}
}


/*////////////////////////////////////////////////////////////////////////////////
   Probudi vojaka k zivotu
*/
void GSoldier::RaiseFromDead(GMap *map, int vybaven){
	int i;

	m_angle  = rand()%256;

	m_angleRad = m_angle*DPI/255;

	m_speed = 0;

	int a, b;
	map->GetFreePos(&a, &b, 20);
	m_x = a;
	m_y = b;

	m_health = Sparams[m_ID].health;
	m_shield = Sparams[m_ID].shields;

	for(i = 0; i < MAX_STREL; i++) {
		m_naboje[i] = (Nparams[i].strel*vybaven)/100;
		m_narust[i] = Nparams[i].raise;
	}

	m_lasthit = -1;   
}

//////////////////////////////////////////////////////////////////////////////////
//                              NARAZOVE FUNKCE
//////////////////////////////////////////////////////////////////////////////////


bool GSoldier::HitBy(int demage, int who)
{
	m_shield -= demage;
	if (m_shield < 0) {
		m_health += m_shield;
		m_shield = 0;
		m_a_krev = MAX_KREV_ANIM-1;
	}
	else  {
		m_a_shield = MAX_STIT_ANIM-1;
	}
	if (m_health <= 0) return true;

	return false;
}

void GSoldier::CompMove(GRun *run, int ID)
{
	m_AI.Move(run, ID);
}


