#ifndef _H_HBITMAP_
#define _H_HBITMAP_

#include <allegro.h>

class HPalette {
private:
 RGB *pal;  // vlastni paleta

public:
 HPalette() {pal= new RGB[255];}
 ~HPalette() {delete[] pal;}
 HPalette(const HPalette& pl) {pal= new RGB[255]; memcpy(pal,pl.pal,768);}

 HPalette& operator= (const HPalette& pl);
 operator RGB* () { return pal;}

 void set_palette();
 void get_palette();
};

class HBitmap {
private:
 BITMAP *bmp;  // objekt bitmapy

public:
 HBitmap() {bmp= NULL;}
 HBitmap(int _x,int _y) {bmp= ::create_bitmap(_x,_y);}
 HBitmap(const char* str,HPalette &pl);
 HBitmap(const HBitmap& bm);
 ~HBitmap() {if (bmp) {destroy_bitmap(bmp); bmp= NULL;} };


// int GetW() {if (bmp) return bmp->w; else return 0;}
// int GetH() {if (bmp) return bmp->h; else return 0;}

 HBitmap& operator= (const HBitmap& bm);
 operator BITMAP* () { return bmp;}
 int operator== (const HBitmap& bm);
 int operator== (BITMAP* bm);


 int load_bitmap(const char* str, HPalette &pl);
 void create_bitmap(int w,int h);

 void Set(BITMAP* bm);
 void Set(HBitmap bm);
 void Unset();

 // blitting
 void clear();
 void clear_to_color(int color);
 void blit(BITMAP* dest,int source_x, int source_y, int dest_x, int dest_y, int width, int height);
 void blit(HBitmap& dest,int source_x, int source_y, int dest_x, int dest_y, int width, int height);
 void blit(BITMAP* dest,int source_x=0, int source_y=0, int dest_x=0, int dest_y=0);
 void blit(HBitmap& dest,int source_x=0, int source_y=0, int dest_x=0, int dest_y=0);
 void masked_blit(BITMAP* dest,int source_x, int source_y, int dest_x, int dest_y, int width, int height);
 void masked_blit(HBitmap& dest,int source_x, int source_y, int dest_x, int dest_y, int width, int height);
 void masked_blit(BITMAP* dest,int source_x=0, int source_y=0, int dest_x=0, int dest_y=0);
 void masked_blit(HBitmap& dest,int source_x=0, int source_y=0, int dest_x=0, int dest_y=0);

 // drawing primitives
 void putpixel(int _x,int _y,int col);
 void _putpixel(int _x,int _y,unsigned char col);
 int getpixel(int _x, int _y);
 unsigned char _getpixel(int _x, int _y);
 void vline(int x, int y1, int y2, int color);
 void hline(int x1, int y, int x2, int color);
 void line(int x1, int y1, int x2, int y2, int color);
 void triangle(int x1,int y1,int x2,int y2,int x3,int y3, int color);
 void polygon(int vertices, int *points, int color);
 void rect(int x1, int y1, int x2, int y2, int color);
 void rectfill(int x1, int y1, int x2, int y2, int color);
 void circle(int x, int y, int radius, int color);
 void circlefill(int x, int y, int radius, int color);
 void ellipse(int x, int y, int rx, int ry, int color);
 void ellipsefill(int x, int y, int rx, int ry, int color);
 void do_arc(int x,int y, fixed a1,fixed a2, int r,int d, void (*proc)());
 void arc(int x,int y, fixed ang1,fixed ang2, int r, int color);
 void spline(int points[8], int color);
 void floodfill(int x, int y, int color);

 // ostatni
 int is_linear() {return is_linear_bitmap(bmp); }
 int is_planar() {return is_planar_bitmap(bmp);}
 int is_memory() {return is_memory_bitmap(bmp);}
 int is_screen() {return is_screen_bitmap(bmp);}
 int is_sub() {return is_sub_bitmap(bmp);}
};

#endif // #ifndef _H_HBITMAP_
