/*  Hippo Games - 2000 (C) - http://www.prog.cz/hippo/
 *    __________________________________________________
 *   /\     __  __    _____   ______   ______   ______  \
 *   \ \   /\ \|\ \  /\_  _\ /\  __ \ /\  __ \ /\  __ \  \
 *    \ \  \ \ \_\ \ \//\ \/ \ \ \_\ \\ \ \_\ \\ \ \ \ \  \
 *     \ \  \ \  __ \  \ \ \  \ \  __/ \ \  __/ \ \ \ \ \  \
 *      \ \  \ \ \/\ \  \_\ \_ \ \ \/   \ \ \/   \ \ \_\ \  \
 *       \ \  \ \_\ \_\ /\____\ \ \_\    \ \_\    \ \_____\  \
 *        \ \  \/_/\/_/ \/____/  \/_/     \/_/     \/_____/   \
 *         \ \_________________________________________________\
 *          \/_________________________________________________/
 *                           
 *  
 *  Sub     : Implementation of HScreen class, wraper for Allegro screen
 *
 *  File    : HScreen.cpp
 *  
 *  Projekt : HAllegro
 *  
 *  Autor   : Bernard Lidicky [berny@prog.cz]
 *
 *  Datum   : 25.4.2000
 *
 */      

#include "stdh.h"
#include <string.h>
#include <stdio.h>

/* HScreen::GetStav() : 
 * NULLing all
 */
HScreen::HScreen()
{
	m_stav = 0;
	m_page1  = NULL;
	m_page2  = NULL;
	m_buffer = NULL;
	m_back   = NULL;
	m_w = 0;
	m_h = 0;
	m_mouse = false;
	m_saves = 0;
	m_fps = 0;
	m_last = 0;
}

/* HScreen::GetStav() : 
 * empty destructor
 */
HScreen::~HScreen() 
{

}

/* HScreen::GetStav() : 
 * returns pointer to visile BITMAP
 */
BITMAP* HScreen::GetVisible()
{
	if (m_stav == H_SCREEN_2PAGES)
		if (m_page1 == m_back) return m_page2;
		else return m_page1;
	else return screen;
}

/* HScreen::GetStav() : 
 * returns m_stav
 */
int HScreen::GetStav()
{
	return m_stav;
}

/* HScreen::Init(int w, int h, int bpp, int card) : 
 * sets set_gfx_mode and set_color_depth, creates m_back and makes black screen
 */
bool HScreen::Init(int w, int h, int bpp, int card)
{
	// chnges resolution
	set_color_depth(bpp);
   if(set_gfx_mode(card, w, h, 0, 0) != 0)
		return false; // failed :(

	m_w = w;
	m_h = h;
	// try to make video pages for flipping 
   m_page1 = create_video_bitmap(m_w, m_h);
   m_page2 = create_video_bitmap(m_w, m_h);

	// if it fails, we can use just simple back buffer
   if ((m_page1 == NULL) || (m_page2 == NULL)) {
	   m_buffer = create_bitmap(m_w, m_h);
		if (m_buffer == NULL)
			return false;   // failed in creating back buffer
		m_stav = H_SCREEN_BUFFER;
		m_back = m_buffer;
   }
	else {
		m_stav = H_SCREEN_2PAGES;
		m_back = m_page2;
	}

	// filling screen by black..
	clear(m_back);
	Flip();
	clear(m_back);
	Flip();
	clear(m_back);

	set_color_conversion(COLORCONV_TOTAL);

	// it's done :)
	return true;
}

/* HScreen::Flip() : 
 * makes visible what is in m_back
 */
int HScreen::Flip(bool VSYNC)
{
	if (m_stav == 0) return 0;
	// update FPS
//	if (m_last == 0) m_last = timeGetTime();
	DWORD tm = timeGetTime();
	m_fps = 1000.0/(tm-m_last);
	m_last = tm;

	if(key[KEY_C]) SaveScreen(!key[KEY_LSHIFT], NULL);

	if (m_mouse) 
		draw_sprite(m_back, b_mouse[act_mouse], mouse_x-mpos[act_mouse][0], mouse_y-mpos[act_mouse][1]);

	char s[50];
	sprintf(s,"%.2f FPS",m_fps);
	textout(m_back, font, s, DF_X-70, 25, makecol(200,200,200));



	if (VSYNC) vsync();
	// page flipping mode
	if (m_stav == H_SCREEN_2PAGES) {
		show_video_bitmap(m_back);
		if (m_back == m_page1) m_back = m_page2;      
		else m_back = m_page1;
   }
	// 1 back buffer mode
	if (m_stav == H_SCREEN_BUFFER) {
		acquire_screen(); // arychli praci s obrazovkou
		draw_sprite(screen, m_buffer, 0, 0);
		release_screen();
	}

	show_mouse(NULL);                   // s tou mysi si nejsem jistej

	if (key[KEY_F1]) Repare();
	if (key[KEY_F2]) ToBack();

	clear(m_back);

	return 1;
}

/* HScreen::Destroy() : 
 * destroy all alocated bitmaps
 */
void HScreen::Destroy()
{
	if (m_page1 != NULL) {
		destroy_bitmap(m_page1);
		m_page1 = NULL;
	}

	if (m_page2 != NULL) {
		destroy_bitmap(m_page2);
		m_page2 = NULL;
	} 

	if (m_buffer != NULL) {
		destroy_bitmap(m_buffer);
		m_buffer = NULL;
	}

	m_back = NULL;
	m_stav = 0;
}

/* HScreen::SaveScreen(char *file) : 
 * ulozi obrazovku do souboru
 * small - zda se ma ulozit 320*240, ci 640 na 480
 */
void HScreen::SaveScreen(bool small, char *file)
{
	char mfile[50];
	if (file != NULL) 
		strcpy(mfile, file);
	else 
		sprintf(mfile,"myal%.4d.bmp",m_saves++);

   BITMAP *bmp;
   PALETTE pal;
	int dx = 320, dy = 240;
	if(small == false) {
		dx = 640;
		dy = 480;
	}

   get_palette(pal);
   bmp = create_bitmap(dx, dy);
	stretch_blit(m_back, bmp, 0, 0, SCREEN_W, SCREEN_H, 0, 0, dx, dy);
   save_bitmap(mfile, bmp, pal);
   destroy_bitmap(bmp);
}

/* HScreen::Repare() : 
 * pri alt+tab se muze porouchat jeden surface - snaha o opraveni
 */
void HScreen::Repare()
{
//	if (m_stav != H_SCREEN_2PAGES) return;

	if (m_page1 != NULL) destroy_bitmap(m_page1);
	if (m_page2 != NULL) destroy_bitmap(m_page2);
	m_page1 = create_video_bitmap(m_w, m_h);
	m_page2 = create_video_bitmap(m_w, m_h);
	m_back = m_page1;
	
	if (m_page1 == NULL || m_page2 == NULL) ToBack();
}

/* HScreen::Repare() : 
 * prepne do modu s zadni bitmapou
 */
void HScreen::ToBack()
{
	if (m_buffer != NULL) destroy_bitmap(m_buffer);
   m_buffer = create_bitmap(m_w, m_h);
	m_stav = H_SCREEN_BUFFER;
	m_back = m_buffer;
}
