//////////////////////////////////////////////////////////////////////////
//
// THE
//     ##   ## ##   ## ##  ## #####  #### ##   ##
//    #### ## ##   ## ##  ## ##     ##  ## ####
//   ## #### ##   ## #####  #####  ##  ##  ##
//  ##   ## ##   ## ##  ## ##     ##  ## ## ##
// ##   ##  #####  ##  ## #####  ##### ##    ##
//												LIBRARY
//
// Programming   : Lasse Skyum (skyum@post5.tele.dk)
//
//////////////////////////////////////////////////////////////////////////

#include "stdh.h"
#include "MultiPlay.h"

NDX_Connect::NDX_Connect()
{
	CoInitialize(NULL);
	lpDP=NULL;
	NumConnections=0;
	NumGames=0;
	NumPlayers=0;
	PlayerStorage=0;
	GameStorage=0;
	RBufferSize=0;
	MessageSize=0;
	RBuffer=malloc(0);
	ConnectionData=NULL;
	Connections=NULL;
	Players=(NetPlayer*)malloc(0);
	Games=(NetGame*)malloc(0);
	PSEHandle=0;
	GSEHandle=0;
	SetGuarenteed(false);
	UseCheckSum=false;
	CheckSumError=false;
}

NDX_Connect::~NDX_Connect()
{
	FREE(ConnectionData);
	RemoveGameScanEvent();
	RemovePlayerScanEvent();
	CloseConnection();
	CoUninitialize();
}

bool WINAPI EnumConnection_Count(LPCGUID lpguidSP,LPVOID lpConnection,DWORD dwConnectionSize,LPCDPNAME lpName,DWORD dwFlags,LPVOID lpContext)
{
	*(int*)((int*)lpContext)+=1;
	return true;
}

bool WINAPI EnumConnection(LPCGUID lpguidSP,LPVOID lpConnection,DWORD dwConnectionSize,LPCDPNAME lpName,DWORD dwFlags,LPVOID lpContext)
{
	NDX_Connect *TC=(NDX_Connect*)lpContext;
	LPDIRECTPLAY4 tlpDP;
	if(FAILED(CoCreateInstance(CLSID_DirectPlay,NULL,CLSCTX_ALL,IID_IDirectPlay4A,(LPVOID*)&tlpDP)))return false;
	if(FAILED(tlpDP->InitializeConnection(lpConnection,0)))
	{
		tlpDP->Release();
		return true;
	}
	tlpDP->Release();
	// Save connection name
	strcpy(TC->Connections[TC->NumConnections].Name,lpName->lpszShortNameA);

	// Save connection GUID
	TC->Connections[TC->NumConnections].guid=*lpguidSP;

	// Save connection data
	TC->Connections[TC->NumConnections].data=malloc(dwConnectionSize);
	memcpy(TC->Connections[TC->NumConnections].data,lpConnection,dwConnectionSize);

	// Increase connection amount
	TC->NumConnections++;
	return true;
}

bool WINAPI EnumSession(LPDPSESSIONDESC2 lpDPGameDesc,LPDWORD lpdwTimeOut,DWORD dwFlags,LPVOID lpContext)
{
	NDX_Connect *TC=(NDX_Connect*)lpContext;
	if(dwFlags&DPESC_TIMEDOUT)return false;
	if(TC->NumGames==TC->GameStorage)
	{
		TC->GameStorage++;
		TC->Games=(NDX_Connect::NetGame*)realloc(TC->Games,sizeof(NDX_Connect::NetGame)*TC->GameStorage);
	}
	strcpy(TC->Games[TC->NumGames].Name,(LPSTR)lpDPGameDesc->lpszSessionNameA);
	TC->Games[TC->NumGames].guid=lpDPGameDesc->guidInstance;
	TC->NumGames++;
	return true;
}

bool WINAPI EnumPlayers(DPID pidID,DWORD dwPlayerType,LPCDPNAME lpName,DWORD dwFlags,LPVOID lpContext)
{
	NDX_Connect *TC=(NDX_Connect*)lpContext;
	if(TC->NumPlayers==TC->PlayerStorage)
	{
		TC->PlayerStorage++;
		TC->Players=(NDX_Connect::NetPlayer*)realloc(TC->Players,sizeof(NDX_Connect::NetPlayer)*TC->PlayerStorage);
	}
	strcpy(TC->Players[TC->NumPlayers].Name,lpName->lpszShortNameA);
	TC->Players[TC->NumPlayers].dpid=pidID;
	TC->NumPlayers++;
	return true;
}

bool NDX_Connect::Create(LPGUID g)
{
	//TODO: tohle je jen testovaci
//	return false;

	guid=g;
	if(lpDP!=NULL)CloseConnection();
	if(FAILED(CoCreateInstance( CLSID_DirectPlay,NULL,CLSCTX_ALL,IID_IDirectPlay4A,(LPVOID*)&lpDP))) {
		allegro_message("CoCreateInstance failed ;(");
		return false;
	}

	long ret;
	ret = CoCreateInstance( CLSID_DirectPlay,NULL,CLSCTX_ALL,IID_IDirectPlay4A,(LPVOID*)&lpDP);
	if(FAILED(ret)) {
		switch (ret) {
		case REGDB_E_CLASSNOTREG :
			allegro_message("CoCreateInstance failed - ne vsechny nerze serveru registrovany");
			if(FAILED(CoCreateInstance( CLSID_DirectPlay,NULL, CLSCTX_REMOTE_SERVER ,IID_IDirectPlay4A,(LPVOID*)&lpDP))) {
				allegro_message("Sorry.");
				return false;
			}
			break;
		case CLASS_E_NOAGGREGATION :
			allegro_message("CoCreateInstance failed - chybi agregat ?");
			return false;
			break;
		default :
			allegro_message("CoCreateInstance failed - neznama chyba ");
			return false;
		}
	}

	int Count=0;
	if(FAILED(lpDP->EnumConnections(guid,(LPDPENUMCONNECTIONSCALLBACK)EnumConnection_Count,&Count,0))) {
		allegro_message("EnumConnection_Count failed ;(");
		return false;
	}
	NumConnections=0;
	Connections=(NetConnection*)malloc(sizeof(NetConnection)*Count);
	if(FAILED(lpDP->EnumConnections(guid,(LPDPENUMCONNECTIONSCALLBACK)EnumConnection,this,0))) {
		allegro_message("EnumConnection failed ;(");
		return false;
	}
	return true;
}

bool NDX_Connect::ScanForGames(int TimeOut,bool Async)
{
	DWORD ASYNC=Async?DPENUMSESSIONS_ASYNC:0;
	DPSESSIONDESC2 dpdesc;
	ZeroMemory(&dpdesc,sizeof(dpdesc));
	dpdesc.dwSize=sizeof(dpdesc);
	dpdesc.guidApplication=*guid;

	NumGames=0;
	if(FAILED(lpDP->EnumSessions(&dpdesc,TimeOut,(LPDPENUMSESSIONSCALLBACK2)EnumSession,this,ASYNC|DPENUMSESSIONS_AVAILABLE)))return false;
	return true;
}


bool NDX_Connect::JoinGame(int GameNumber,LPSTR PlayerName)
{
	DPSESSIONDESC2 dpdesc;
	DPNAME dpName;
	ZeroMemory(&dpdesc,sizeof(dpdesc));
	dpdesc.dwSize=sizeof(dpdesc);
	dpdesc.guidInstance=Games[GameNumber].guid;
	dpdesc.dwFlags=0;
	if(FAILED(lpDP->Open(&dpdesc,DPOPEN_JOIN)))return false;
	ZeroMemory(&dpName,sizeof(dpName));
	dpName.dwSize=sizeof(dpName);
	dpName.lpszShortNameA=PlayerName;
	if(FAILED(lpDP->CreatePlayer(&LocalDPID,&dpName,NULL,NULL,0,0)))return false;
	RemoveGameScanEvent();
	return true;
}

bool NDX_Connect::CreateGame(LPSTR Name,LPSTR PlayerName,int MaxPlayers)
{
	DPSESSIONDESC2 dpdesc;
	DPNAME dpName;
	ZeroMemory(&dpdesc,sizeof(dpdesc));
	dpdesc.dwSize=sizeof(dpdesc);
	dpdesc.guidApplication=*guid;
	dpdesc.dwFlags=DPSESSION_MIGRATEHOST|DPSESSION_KEEPALIVE;
	dpdesc.lpszSessionNameA=Name;
	dpdesc.dwMaxPlayers=MaxPlayers;
	if(FAILED(lpDP->Open(&dpdesc,DPOPEN_CREATE)))return false;
	ZeroMemory(&dpName,sizeof(dpName));
	dpName.dwSize=sizeof(dpName);
	dpName.lpszShortNameA=PlayerName;
	if(FAILED(lpDP->CreatePlayer(&LocalDPID,&dpName,NULL,NULL,0,DPPLAYER_SERVERPLAYER)))return false;
	MaxPlrs=MaxPlayers;
	RemoveGameScanEvent();
	return true;
}

bool NDX_Connect::Connect(int CNum)
{
	if(FAILED(lpDP->InitializeConnection(Connections[CNum].data,0)))return false;
	return true;
}

bool NDX_Connect::ScanForPlayers()
{
	NumPlayers=0;
	if(FAILED(lpDP->EnumPlayers(NULL,(LPDPENUMPLAYERSCALLBACK2)EnumPlayers,this,DPENUMPLAYERS_ALL)))return false;
	return true;
}

bool NDX_Connect::CloseGame()
{
	if(lpDP==NULL)return false;
	if(FAILED(lpDP->DestroyPlayer(LocalDPID)))return false;
	if(FAILED(lpDP->Close()))return false;
	RemovePlayerScanEvent();
	return true;
}

bool NDX_Connect::Receive()
{
	DWORD nBytes;
	HRESULT dprval;
	do
	{
		nBytes=RBufferSize;
		dprval=lpDP->Receive(&FromDPID,&ToDPID,DPRECEIVE_ALL,RBuffer,&nBytes);
		if(dprval==DPERR_BUFFERTOOSMALL)
		{
			RBuffer=realloc(RBuffer,nBytes);
			RBufferSize=nBytes;
		}
	}while(dprval==DPERR_BUFFERTOOSMALL);
	MessageSize=nBytes;

	if(dprval!=DP_OK)return false;

	if(UseCheckSum&&FromDPID!=DPID_SYSMSG)
	{
		MessageSize-=sizeof(DWORD);
		DWORD Sum;
		memcpy(&Sum,(char*)((char*)RBuffer+MessageSize),sizeof(DWORD));
		if(Sum!=CheckSum(RBuffer,MessageSize))
		{
			CheckSumError=true;
			return false;
		}
	}
	return true;
}
 
NDXERR NDX_Connect::Send(DPID To,LPVOID Data,DWORD DataSize)
{
	if(!UseCheckSum)
	{
		HRESULT rval=lpDP->Send(LocalDPID,To,GUARANTEED,Data,DataSize);
		if(rval==DPERR_BUSY)return NDXERR_NETBUSY;
		if(rval!=DP_OK)return NDXERR_SENDMSGERR;
	}else
	{
		char *p=(char*)malloc(DataSize+sizeof(DWORD));
		memcpy(p,Data,DataSize);
		DWORD Sum=CheckSum(Data,DataSize);
		memcpy(p+DataSize,&Sum,sizeof(DWORD));
		HRESULT rval=lpDP->Send(LocalDPID,To,GUARANTEED,p,DataSize+sizeof(DWORD));
		if(rval==DPERR_BUSY)return NDXERR_NETBUSY;
		if(rval!=DP_OK)return NDXERR_SENDMSGERR;
	}
	return NDXERR_OK;
}

NDXERR NDX_Connect::Send(DPID To,DWORD dwType)
{
	if(!UseCheckSum)
	{
		HRESULT rval=lpDP->Send(LocalDPID,To,GUARANTEED,&dwType,4);
		if(rval==DPERR_BUSY)return NDXERR_NETBUSY;
		if(rval!=DP_OK)return NDXERR_SENDMSGERR;
	}else
	{
		char *p=(char*)malloc(sizeof(DWORD)*2);
		memcpy(p,&dwType,sizeof(DWORD));
		DWORD Sum=CheckSum(&dwType,sizeof(DWORD));
		memcpy(p+sizeof(DWORD),&Sum,sizeof(DWORD));
		HRESULT rval=lpDP->Send(LocalDPID,To,GUARANTEED,p,sizeof(DWORD)*2);
		if(rval==DPERR_BUSY)return NDXERR_NETBUSY;
		if(rval!=DP_OK)return NDXERR_SENDMSGERR;
	}
	return NDXERR_OK;
}

int NDX_Connect::FindSPGUID(GUID spguid)
{
	for(int n=0;n<NumConnections;n++)
	{
		if(Connections[n].guid==spguid)return n;
	}
	return -1;
}

bool NDX_Connect::ConnectIPX()
{
	LPDIRECTPLAYLOBBY3A lpDPL;
	DPCOMPOUNDADDRESSELEMENT CElement;
	DWORD AddressSize;
	if(FAILED(CoCreateInstance(CLSID_DirectPlayLobby,NULL,CLSCTX_ALL,IID_IDirectPlayLobby3A,(LPVOID*)&lpDPL))) {
		allegro_message("CoCreateInstance failed :(");
		return false;
	}

	CElement.guidDataType=DPAID_ServiceProvider;
	CElement.dwDataSize=sizeof(GUID);
	CElement.lpData=(LPVOID)&DPSPGUID_IPX;

	lpDPL->CreateCompoundAddress((DPCOMPOUNDADDRESSELEMENT*)&CElement,1,NULL,&AddressSize);
	FREE(ConnectionData);
	ConnectionData=malloc(AddressSize);
	if(ConnectionData==NULL)
	{
		lpDPL->Release();
		return false;
	}
	lpDPL->CreateCompoundAddress((DPCOMPOUNDADDRESSELEMENT*)&CElement,1,ConnectionData,&AddressSize);
	if(FAILED(lpDP->InitializeConnection(ConnectionData,0)))
	{
		lpDPL->Release();
		allegro_message("InitializeConnection failed.");
		return false;
	}
	lpDPL->Release();
	return true;
}

bool NDX_Connect::ConnectTCPIP(char * IPAddress)
{
	LPDIRECTPLAYLOBBY3A lpDPL;
	DPCOMPOUNDADDRESSELEMENT CElement[2];
	DWORD AddressSize;
	if(FAILED(CoCreateInstance(CLSID_DirectPlayLobby,NULL,CLSCTX_ALL,IID_IDirectPlayLobby3A,(LPVOID*)&lpDPL)))return false;

	CElement[0].guidDataType=DPAID_ServiceProvider;
	CElement[0].dwDataSize=sizeof(GUID);
	CElement[0].lpData=(LPVOID)&DPSPGUID_TCPIP;
	CElement[1].guidDataType=DPAID_INet;
	CElement[1].dwDataSize=strlen(IPAddress)+1;
	CElement[1].lpData=IPAddress;

	lpDPL->CreateCompoundAddress((DPCOMPOUNDADDRESSELEMENT*)&CElement,2,NULL,&AddressSize);
	FREE(ConnectionData);
	ConnectionData=malloc(AddressSize);
	if(ConnectionData==NULL)
	{
		lpDPL->Release();
		return false;
	}
	lpDPL->CreateCompoundAddress((DPCOMPOUNDADDRESSELEMENT*)&CElement,2,ConnectionData,&AddressSize);
	if(FAILED(lpDP->InitializeConnection(ConnectionData,0)))
	{
		lpDPL->Release();
		return false;
	}
	lpDPL->Release();
	return true;
}

bool NDX_Connect::ConnectMODEM(char * Phone)
{
	LPDIRECTPLAYLOBBY3A lpDPL;
	DPCOMPOUNDADDRESSELEMENT CElement[2];
	DWORD AddressSize;
	if(FAILED(CoCreateInstance(CLSID_DirectPlayLobby,NULL,CLSCTX_ALL,IID_IDirectPlayLobby3A,(LPVOID*)&lpDPL)))return false;

	CElement[0].guidDataType=DPAID_ServiceProvider;
	CElement[0].dwDataSize=sizeof(GUID);
	CElement[0].lpData=(LPVOID)&DPSPGUID_MODEM;
	CElement[1].guidDataType=DPAID_Phone;
	CElement[1].dwDataSize=strlen(Phone)+1;
	CElement[1].lpData=Phone;

	lpDPL->CreateCompoundAddress((DPCOMPOUNDADDRESSELEMENT*)&CElement,2,NULL,&AddressSize);
	ConnectionData=malloc(AddressSize);	
	if(ConnectionData==NULL)
	{
		lpDPL->Release();
		return false;
	}
	lpDPL->CreateCompoundAddress((DPCOMPOUNDADDRESSELEMENT*)&CElement,2,ConnectionData,&AddressSize);
	if(FAILED(lpDP->InitializeConnection(ConnectionData,0)))
	{
		lpDPL->Release();
		return false;
	}
	lpDPL->Release();
	return true;
}

bool NDX_Connect::CloseConnection()
{
	CloseGame();
	for(int n=0;n<NumConnections;n++)
	{
		free(Connections[n].data);
	}
	FREE(Connections);
	FREE(Games);
	FREE(Players);
	FREE(ConnectionData);
	if (lpDP != NULL) {
		lpDP->Release();
		lpDP = NULL;
	FREE(RBuffer);
	}
	return true;
}

bool NDX_Connect::ConnectSERIAL(DWORD ComPort,DWORD BaudRate)
{
	LPDIRECTPLAYLOBBY3A lpDPL;
	DPCOMPOUNDADDRESSELEMENT CElement[2];
	DPCOMPORTADDRESS ComPortAddress;
	DWORD AddressSize;
	if(FAILED(CoCreateInstance(CLSID_DirectPlayLobby,NULL,CLSCTX_ALL,IID_IDirectPlayLobby3A,(LPVOID*)&lpDPL)))return false;

	ComPortAddress.dwComPort=ComPort;
	ComPortAddress.dwBaudRate=BaudRate;
	ComPortAddress.dwStopBits=ONESTOPBIT;
	ComPortAddress.dwParity=NOPARITY;
	ComPortAddress.dwFlowControl=DPCPA_RTSFLOW;

	CElement[0].guidDataType=DPAID_ServiceProvider;
	CElement[0].dwDataSize=sizeof(GUID);
	CElement[0].lpData=(LPVOID)&DPSPGUID_SERIAL;
	CElement[1].guidDataType=DPAID_ComPort;
	CElement[1].dwDataSize=sizeof(ComPortAddress);
	CElement[1].lpData=&ComPortAddress;

	lpDPL->CreateCompoundAddress((DPCOMPOUNDADDRESSELEMENT*)&CElement,2,NULL,&AddressSize);
	ConnectionData=malloc(AddressSize);
	if(ConnectionData==NULL)
	{
		lpDPL->Release();
		return false;
	}
	lpDPL->CreateCompoundAddress((DPCOMPOUNDADDRESSELEMENT*)&CElement,2,ConnectionData,&AddressSize);
	if(FAILED(lpDP->InitializeConnection(ConnectionData,0)))
	{
		lpDPL->Release();
		return false;
	}
	lpDPL->Release();
	return true;
}

int NDX_Connect::GetMessageCount()
{
	DWORD MessageCount;
	lpDP->GetMessageCount(LocalDPID,&MessageCount);
	return MessageCount;
}

void CALLBACK PlayerScanEvent(UINT uID,UINT uMsg,void *dwUser,DWORD dw1,DWORD dw2)
{
	NDX_Connect *TC=(NDX_Connect*)dwUser;
	TC->ScanForPlayers();
}

NDXERR NDX_Connect::SetPlayerScanEvent(int MiliSec)
{
	RemovePlayerScanEvent();
	TIMECAPS TimeCaps;
	timeGetDevCaps(&TimeCaps,sizeof(TimeCaps));
	if(MiliSec<int(TimeCaps.wPeriodMin))MiliSec=TimeCaps.wPeriodMin;
	if(MiliSec>int(TimeCaps.wPeriodMax))MiliSec=TimeCaps.wPeriodMax;

	PSEHandle=timeSetEvent(MiliSec,0,(LPTIMECALLBACK)PlayerScanEvent,(DWORD)this,TIME_PERIODIC);
	if(PSEHandle)return NDXERR_OK;
	return NDXERR_EVENT;
}

void CALLBACK GameScanEvent(UINT uID,UINT uMsg,void *dwUser,DWORD dw1,DWORD dw2)
{
	NDX_Connect *TC=(NDX_Connect*)dwUser;
	TC->ScanForGames(0,true);
}

NDXERR NDX_Connect::SetGameScanEvent(int MiliSec)
{
	RemoveGameScanEvent();
	TIMECAPS TimeCaps;
	timeGetDevCaps(&TimeCaps,sizeof(TimeCaps));
	if(MiliSec<int(TimeCaps.wPeriodMin))MiliSec=TimeCaps.wPeriodMin;
	if(MiliSec>int(TimeCaps.wPeriodMax))MiliSec=TimeCaps.wPeriodMax;
	
	GSEHandle=timeSetEvent(MiliSec,0,(LPTIMECALLBACK)GameScanEvent,(DWORD)this,TIME_PERIODIC);
	if(GSEHandle)return NDXERR_OK;
	return NDXERR_EVENT;
}

NDXERR NDX_Connect::RemovePlayerScanEvent()
{
	if(PSEHandle)timeKillEvent(PSEHandle);
	PSEHandle=0;
	return NDXERR_OK;
}

NDXERR NDX_Connect::RemoveGameScanEvent()
{
	if(GSEHandle)timeKillEvent(GSEHandle);
	GSEHandle=0;
	return NDXERR_OK;
}

void NDX_Connect::SetGuarenteed(bool OnOff)
{
	if(OnOff)GUARANTEED=DPSEND_GUARANTEED;else GUARANTEED=0;
}

bool NDX_Connect::SetRBufferSize(int Size)
{
	RBuffer=realloc(RBuffer,Size);
	RBufferSize=Size;
	if(RBuffer==NULL)return false;
	return true;
}

bool NDX_Connect::HideGame()
{
	DWORD DataSize;
	void *Data;
	DPSESSIONDESC2 dpdesc;
	ZeroMemory(&dpdesc,sizeof(dpdesc));
	lpDP->GetSessionDesc(NULL,&DataSize);
	Data=malloc(DataSize);
	lpDP->GetSessionDesc(Data,&DataSize);
	memcpy(&dpdesc,Data,sizeof(dpdesc));
	free(Data);
	dpdesc.dwSize=sizeof(dpdesc);
	dpdesc.dwFlags|=DPSESSION_JOINDISABLED;
	if(lpDP->SetSessionDesc(&dpdesc,0)!=DP_OK)return false;
	return true;
}

bool NDX_Connect::UnHideGame()
{
	DWORD DataSize;
	void *Data;
	DPSESSIONDESC2 dpdesc;
	ZeroMemory(&dpdesc,sizeof(dpdesc));
	lpDP->GetSessionDesc(NULL,&DataSize);
	Data=malloc(DataSize);
	lpDP->GetSessionDesc(Data,&DataSize);
	memcpy(&dpdesc,Data,sizeof(dpdesc));
	free(Data);
	dpdesc.dwSize=sizeof(dpdesc);
	dpdesc.dwFlags&=DPSESSION_JOINDISABLED^0xFFFFFFFF;
	if(lpDP->SetSessionDesc(&dpdesc,0)!=DP_OK)return false;
	return true;
}

int NDX_Connect::GetPackageSize()
{
	DPCAPS dpcaps;
	lpDP->GetCaps(&dpcaps,GUARANTEED);
	return dpcaps.dwMaxBufferSize;
}

DWORD NDX_Connect::CheckSum(void * p, DWORD s)
{
	DWORD Sum=0;
	unsigned char *bp=(unsigned char*)p;
	for(DWORD n=0;n<s;n++)
	{
		Sum+=*bp++;
	}
	return Sum;
}
